/**
 * BlackBerry Limited proprietary and confidential.
 * Do not reproduce without permission in writing.
 * Copyright (c) 2017 BlackBerry Ltd.
 * All rights reserved.
 */
package com.rim.platform.samples.rest.oidc;

import java.io.IOException;
import java.net.URISyntaxException;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.rim.platform.samples.exception.SampleException;

/**
 * Provides access to the OIDC discovery document
 */
public class OidcDiscoveryDocumentService
{
    /**
     * URI pattern for the discovery document
     */
    private static final String OP_WELL_KNOWN_OPENID_CONFIGURATION = "/op/%s/.well-known/openid-configuration";

    /**
     * Simple logger. See simplelogger.properties to change the logging settings
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(OidcDiscoveryDocumentService.class);

    /**
     * The endpoints available in the discovery document
     */
    public enum EndpointKey
    {
        INITIAL_ACCESS_TOKEN_ENDPOINT("initial_access_token_endpoint"),
        REGISTRATION_ENDPOINT("registration_endpoint"),
        TOKEN_ENDPOINT("token_endpoint");

        private String key;

        EndpointKey(final String key)
        {
            this.key = key;
        }

        public String getKey()
        {
            return key;
        }
    }

    /**
     * Returns the endpoint URL for the passed EndpointKey and tenant Id
     *
     * @param eidUrl
     *            the EID URL to use to get the endpoint URL
     * @param endpointKey
     *            the EndpointKey to to get
     * @param tenantId
     *            the Tenant Id to get the endpoint URL for
     * @return the endpoint URL for the passed EndpointKey and tenant Id
     * @throws IOException
     * @throws SampleException
     */
    public String getEndpointUrl(String eidUrl, EndpointKey endpointKey, String tenantId) throws IOException, SampleException
    {
        // Create the URI for the discovery document URL
        String uri = null;
        if (StringUtils.isBlank(tenantId))
        {
            uri = String.format(OP_WELL_KNOWN_OPENID_CONFIGURATION, "");
        }
        else
        {
            uri = String.format(OP_WELL_KNOWN_OPENID_CONFIGURATION, "tenant/" + tenantId);
        }

        // Create the discovery document URL and get the discovery document
        String discoveryURL = eidUrl + uri;
        String content = getDiscoveryDocument(discoveryURL);

        // Parse the response into a JSON document
        ObjectMapper mapper = new ObjectMapper();
        JsonNode discoveryDocument = mapper.readTree(content);

        // Find the specified value in the response document
        String tokenEndpoint = discoveryDocument.findValue(endpointKey.getKey()).asText();
        LOGGER.debug("Token endpoint url {}", tokenEndpoint);

        //
        return tokenEndpoint;
    }

    /**
     * 
     * @param url
     * @return
     * @throws URISyntaxException
     * @throws IOException
     * @throws SampleException
     */
    private String getDiscoveryDocument(String url) throws IOException, SampleException
    {
        LOGGER.debug("Getting the discovery document from {}", url);

        // Create the HTTP client
        try (CloseableHttpClient httpClient = createHttpClient())
        {
            // Create the GET request
            HttpGet get = new HttpGet(url);

            // Execute the request
            try (CloseableHttpResponse response = httpClient.execute(get))
            {
                // Get and check the status code
                int statusCode = response.getStatusLine().getStatusCode();
                HttpEntity responseEntity = response.getEntity();
                if (statusCode == HttpStatus.SC_OK)
                {
                    // Return the response as a String
                    return EntityUtils.toString(responseEntity);
                }
                else
                {
                    throw new SampleException("Failed to get the discovery document, reason: " + statusCode + ", "
                            + (responseEntity != null ? EntityUtils.toString(responseEntity) : "No content"));
                }
            }
        }
    }

    /**
     * Creates a basic HTTP client
     * 
     * @return the HTTP client
     */
    private CloseableHttpClient createHttpClient()
    {
        return HttpClients.custom().setDefaultRequestConfig(RequestConfig.custom().setCookieSpec(CookieSpecs.STANDARD).build())
                .build();
    }
}
