$tenantGuid 	= "Set-Tenant-Guid-Here"
$eidBaseUrl 	= "https://idp.blackberry.com"
$clientId 		= "Set-Client-Id-Here";
# Get the pfx keystore associated with the private key. You can copy into current working folder
$certFilePath 	= "$pwd\Set-PFX-formatted-keystore"
$password 		= "Set-Keystore-Password"
$scope 			= "MDMBWS.All"
# UEM On-premise API port is 18084 and Cloud API port is 443
$restApiUrl 	= "https://Set-Host-Here:18084/{0}/api/v1/ping" -f $tenantGuid;

function getDiscoveryDocument
{
    $methodName = "getDiscoveryDocument()";
    enterMethod($methodName);

    # Get Discovery document for tenant
    try
    {
        [Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Ssl3, [Net.SecurityProtocolType]::Tls, [Net.SecurityProtocolType]::Tls11, [Net.SecurityProtocolType]::Tls12

        $discoveryUrl = $eidBaseUrl + "/op/tenant/{0}/.well-known/openid-configuration" -f $tenantGuid;

        Write-Output ""
        Write-Output "Getting Discovery Document from $discoveryUrl"

        $result = Invoke-RestMethod -Method GET -Uri $discoveryUrl -ErrorAction STOP

        if ($result.token_endpoint){
            $global:tokenEndpoint = $result.token_endpoint
            Write-Output ""
            Write-Output "Obtained token endpoint: $tokenEndpoint"
        }
    }
    catch
    {
        handleError $_;
    }
}

function getAccessToken
{
    $Certificate = [System.Security.Cryptography.X509Certificates.X509Certificate2]::new($certFilePath, $password)

    # Create JWT timestamp for expiration
    $StartDate = (Get-Date "1970-01-01T00:00:00Z" ).ToUniversalTime()
    $JWTExpirationTimeSpan = (New-TimeSpan -Start $StartDate -End (Get-Date).ToUniversalTime().AddMinutes(2)).TotalSeconds
    $JWTExpiration = [math]::Round($JWTExpirationTimeSpan,0)

    # Create JWT header
    $JWTHeader = @{
        alg = "RS256"
        typ = "JWT"
    }

    # Create JWT payload
    $JWTPayLoad = @{
        aud = $global:tokenEndpoint
        exp = $JWTExpiration
        iss = $clientId
        jti = [guid]::NewGuid()
        sub = $clientId
    }

    # Convert header and payload to base64
    $JWTHeaderToByte = [System.Text.Encoding]::UTF8.GetBytes(($JWTHeader | ConvertTo-Json))
    $EncodedHeader = [System.Convert]::ToBase64String($JWTHeaderToByte)

    $JWTPayLoadToByte =  [System.Text.Encoding]::UTF8.GetBytes(($JWTPayload | ConvertTo-Json))
    $EncodedPayload = [System.Convert]::ToBase64String($JWTPayLoadToByte)

    # Join header and Payload with "." to create a valid (unsigned) JWT
    $JWT = $EncodedHeader + "." + $EncodedPayload

    # Get the private key object of your certificate
    $PrivateKey = $Certificate.PrivateKey

    # Force use of the Enhanced RSA and AES Cryptographic Provider with openssl-generated SHA256 keys
    $enhCsp = [System.Security.Cryptography.RSACryptoServiceProvider]::new().CspKeyContainerInfo
    $cspparams = [System.Security.Cryptography.CspParameters]::new($enhCsp.ProviderType, $enhCsp.ProviderName, $PrivateKey.CspKeyContainerInfo.KeyContainerName)
    $privKey = [System.Security.Cryptography.RSACryptoServiceProvider]::new($cspparams)

    # Define RSA signature and hashing algorithm
    $RSAPadding = [Security.Cryptography.RSASignaturePadding]::Pkcs1
    $HashAlgorithm = [Security.Cryptography.HashAlgorithmName]::SHA256

    # Create a signature of the JWT
    $Signature = [Convert]::ToBase64String(
        $privKey.SignData([System.Text.Encoding]::UTF8.GetBytes($JWT),$HashAlgorithm, $RSAPadding)
    ) -replace '\+','-' -replace '/','_' -replace '='

    # Join the signature to the JWT with "." to create the client assertion
    $JWT = $JWT + "." + $Signature

    # Create a hash with body parameters
    $Body = @{
        client_id = $AppId
        client_assertion = $JWT
        client_assertion_type = "urn:ietf:params:oauth:client-assertion-type:jwt-bearer"
        scope = $scope
        grant_type = "client_credentials"
    }

    $PostSplat = @{
        ContentType = 'application/x-www-form-urlencoded'
        Method = 'POST'
        Body = $Body
        Uri = $global:tokenEndpoint
    }

    Write-Output "Attempting to get access token"
    $result = Invoke-RestMethod @PostSplat
    if ($result.access_token){
            $global:accessToken = $result.access_token
            Write-Output "Received access token:"
            $global:accessToken
    }
}

function invokeRestApi
{
    $methodName = "InvokeRestApi()";
    enterMethod($methodName);

    # Invoke the REST Api with the access token in the Authorization header
    try
    {
        # Add Trust All
        add-type @"
        using System.Net;
        using System.Security.Cryptography.X509Certificates;
        public class TrustAllCertsPolicy : ICertificatePolicy {
            public bool CheckValidationResult(
                ServicePoint srvPoint, X509Certificate certificate,
                WebRequest request, int certificateProblem) {
                return true;
            }
        }
"@
        [System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy
        [Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Ssl3, [Net.SecurityProtocolType]::Tls, [Net.SecurityProtocolType]::Tls11, [Net.SecurityProtocolType]::Tls12

        $authHeader = @{"Authorization" = ("Bearer", $global:accessToken -join " ")}

        Write-Output ""
        Write-Output "Attempting to access Public REST API"
        Write-Output "Request: $restApiUrl"

        $response = Invoke-RestMethod -Method GET -Uri $restApiUrl -Headers $authHeader -ErrorAction STOP

        Write-Output ""
        Write-Output "Received response for rest api: $restApiUrl"
        Write-Output $response
        Write-Output ""
    }
    catch
    {
        handleError $_;
    }

    Write-Output ""
    exitMethod($methodName);
    Write-Output ""
    Write-Output("Complete!");
    exit(0);
}

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command could not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

# Start.
getDiscoveryDocument;
getAccessToken;
InvokeRestApi;
